# -*- coding: utf-8 -*-
# This file is part of quark-sphinx-theme.
# Copyright (c) 2016 Felix Krull <f_krull@gmx.de>
# Released under the terms of the BSD license; see LICENSE.

from __future__ import print_function

from contextlib import contextmanager
from io import open
import os
import shutil
import sys
import tempfile

try:
    # Prefer StringIO on Python 2 so it can cope with str.
    from StringIO import StringIO
except ImportError:
    from io import StringIO

import html5lib
import sphinx
try:
    import tinycss
except ImportError:
    tinycss = None


@contextmanager
def attrs(obj, **kwargs):
    oldvals = {}
    try:
        for k, v in kwargs.items():
            oldval = getattr(obj, k)
            setattr(obj, k, v)
            oldvals[k] = oldval
        yield
    finally:
        for k, oldval in oldvals.items():
            setattr(obj, k, oldval)


@contextmanager
def capture_output():
    stdout = StringIO()
    with attrs(sys, stdout=stdout, stderr=stdout):
        yield stdout


def get_css_errors(filename_or_fobj, encoding=None):
    if hasattr(filename_or_fobj, 'read'):
        source = filename_or_fobj.read()
    else:
        with open(filename_or_fobj, 'r', encoding=encoding) as fobj:
            source = fobj.read()
    parser = tinycss.make_parser()
    css = parser.parse_stylesheet(source)
    lines = source.split('\n')
    errors = []
    for err in css.errors:
        try:
            errors.append('  %s [%s]\n' % (err, lines[err.line - 1]))
        except IndexError:
            errors.append('  %s\n' % err)
    return errors


def run_sphinx(source_dir, build_dir, builder='html', extra_config={}, tags=[],
               sphinx_build='sphinx-build'):
    args = ['sphinx_build', '-b', builder, '-N']
    for k, v in extra_config.items():
        args.extend(['-D', '%s=%s' % (k, v)])
    for tag in tags:
        args.extend(['-t', tag])
    args.extend([source_dir, build_dir])
    with capture_output() as stdout:
        try:
            ret = sphinx.main(args)
        except SystemExit as e:
            ret = e.code
        if ret != 0:
            raise Exception('%s returned non-zero exit status %s\n'
                            '--- Output:\n%s----' %
                            (args, ret, stdout.getvalue()))


class SphinxBuildFixture:
    source_dir = None
    builder = 'html'
    config = {}
    tags = []

    build_dir = None

    @classmethod
    def setUpClass(cls):
        if not cls.source_dir:
            raise Exception('source_dir not set')
        cls.build_dir = tempfile.mkdtemp(prefix='tmp-sphinx-build-test-')
        try:
            run_sphinx(cls.source_dir, cls.build_dir, cls.builder, cls.config,
                       cls.tags)
        except Exception:
            cls.tearDownClass()
            raise

    @classmethod
    def tearDownClass(cls):
        if cls.build_dir:
            shutil.rmtree(cls.build_dir, True)
            cls.build_dir = None

    def read_document(self, name):
        path = os.path.join(self.build_dir, *name.split('/')) + '.html'
        with open(path, 'rb') as f:
            return html5lib.parse(f, namespaceHTMLElements=False)

    def assertHasElement(self, doc, path, n=1):
        self.assertEqual(len(list(doc.findall(path))), n)

    def assertNotElement(self, doc, path):
        self.assertHasElement(doc, path, 0)

    def assertCSSValid(self, filename_or_fobj, encoding=None, auto_skip=True):
        if not tinycss and auto_skip:
            self.skipTest('tinycss not available')
        errors = get_css_errors(filename_or_fobj, encoding)
        self.assertEqual(len(errors), 0, 'CSS errors:\n' + ''.join(errors))

    def assertSphinxCSSValid(self, basename, encoding='utf-8', auto_skip=True):
        self.assertCSSValid(os.path.join(self.build_dir, '_static', basename),
                            encoding=encoding,
                            auto_skip=auto_skip)


def with_document(docname):
    def wrap1(func):
        def wrap2(self, *args, **kwargs):
            func(self, self.read_document(docname), *args, **kwargs)
        return wrap2
    return wrap1


def with_elements(docname, xpath_expr, n=None):
    def wrap1(func):
        def wrap2(self, *args, **kwargs):
            elems = list(self.read_document(docname).findall(xpath_expr))
            if n is not None and len(elems) != n:
                raise AssertionError('expected %s element(s), got %s' %
                                     (n, len(elems)))
            elif len(elems) == 0:
                raise AssertionError('no elements')
            func(self, elems, *args, **kwargs)
        return wrap2
    return wrap1



# -*- coding: utf-8 -*-
# This file is part of quark-sphinx-theme.
# Copyright (c) 2016 Felix Krull <f_krull@gmx.de>
# Released under the terms of the BSD license; see LICENSE.

import os
import sys
import unittest

from quark_sphinx_theme.ext.html_rewrite import (
    create_translator_class, HTMLCompatMixin, BoxesMixin, LiteralBlocksMixin)
from sphinx.writers.html import HTMLTranslator
DIR = os.path.abspath(os.path.dirname(__file__))
sys.path.insert(0, DIR)
from util import SphinxBuildFixture, with_document, with_elements  # noqa


testdoc_html_rewrite = os.path.join(DIR, 'testdoc-html_rewrite')


class TestHTMLRewriteOutputTestCases(SphinxBuildFixture):
    source_dir = testdoc_html_rewrite

    @with_elements('test_pre_block', './/div[@class="highlight"]//pre//span')
    def test_pre_block(self, elems):
        for span in elems:
            self.assertTrue(span.text)

    @with_elements('test_footnote', './/table[@class="docutils footnote"]/..')
    def test_footnote(self, elems):
        for par in elems:
            self.assertEqual(par.tag, 'div')
            self.assertEqual(par.get('class'), '-x-quark-footnote-wrapper')
            self.assertTrue(par.get('id'))
            self.assertFalse(par.find('./table').get('id'))

    @with_elements('test_citation', './/table[@class="docutils citation"]/..')
    def test_citation(self, elems):
        for par in elems:
            self.assertEqual(par.tag, 'div')
            self.assertEqual(par.get('class'), '-x-quark-citation-wrapper')
            self.assertTrue(par.get('id'))
            self.assertFalse(par.find('./table').get('id'))

    @with_elements('test_admonition',
                   './/div[@class="admonition-test admonition"]/../../../..')
    def test_admonition(self, elems):
        for par in elems:
            self.assertEqual(par.tag, 'table')
            self.assertIn('-x-quark-admonition', par.get('class'))
            self.assertIn('-x-quark-box', par.get('class'))

    @with_elements('test_warning',
                   './/div[@class="admonition warning"]/../../../..')
    def test_warning(self, elems):
        for par in elems:
            self.assertEqual(par.tag, 'table')
            self.assertIn('-x-quark-admonition', par.get('class'))
            self.assertIn('-x-quark-warning', par.get('class'))
            self.assertIn('-x-quark-box', par.get('class'))

    @with_elements('test_topic', './/div[@class="topic"]/../../../..')
    def test_topic(self, elems):
        for par in elems:
            self.assertEqual(par.tag, 'table')
            self.assertIn('-x-quark-topic', par.get('class'))
            self.assertIn('-x-quark-box', par.get('class'))

    @with_elements('test_sidebar', './/div[@class="sidebar"]/../../../..')
    def test_sidebar(self, elems):
        for par in elems:
            self.assertEqual(par.tag, 'table')
            self.assertIn('-x-quark-sidebar', par.get('class'))
            self.assertIn('-x-quark-box', par.get('class'))

    @with_elements('test_literal_block',
                   './/div[@class="highlight"]/../../../../..')
    def test_literal_block(self, elems):
        for par in elems:
            self.assertEqual(par.tag, 'table')
            self.assertIn('-x-quark-literal-block', par.get('class'))


class TestHTMLRewriteOutput(TestHTMLRewriteOutputTestCases, unittest.TestCase):
    pass


class TestHTMLCompatOutput(TestHTMLRewriteOutputTestCases, unittest.TestCase):
    tags = ['test_html_compat_alias']


class TestHTMLRewriteFeatures(SphinxBuildFixture, unittest.TestCase):
    source_dir = testdoc_html_rewrite
    config = {
        'quark_html_features': '',
    }

    @with_elements('test_footnote', './/table[@class="docutils footnote"]/..')
    def test_footnote(self, elems):
        for par in elems:
            self.assertNotIn(par.get('class'), '-x-quark-footnote-wrapper')

    @with_elements('test_citation', './/table[@class="docutils citation"]/..')
    def test_citation(self, elems):
        for par in elems:
            self.assertNotIn(par.get('class'), '-x-quark-citation-wrapper')

    @with_elements('test_admonition',
                   './/div[@class="admonition-test admonition"]/../../../..')
    def test_admonition(self, elems):
        for par in elems:
            self.assertNotEqual(par.tag, 'table')
            self.assertNotIn('-x-quark-admonition', par.get('class'))

    @with_elements('test_topic', './/div[@class="topic"]/../../../..')
    def test_topic(self, elems):
        for par in elems:
            self.assertNotEqual(par.tag, 'table')
            self.assertNotIn('-x-quark-topic', par.get('class'))

    @with_elements('test_sidebar', './/div[@class="sidebar"]/../../../..')
    def test_sidebar(self, elems):
        for par in elems:
            self.assertNotEqual(par.tag, 'table')
            self.assertNotIn('-x-quark-sidebar', par.get('class'))

    @with_elements('test_literal_block',
                   './/div[@class="highlight"]/../../../../..')
    def test_literal_block(self, elems):
        for par in elems:
            self.assertNotEqual(par.tag, 'table')
            self.assertNotIn('-x-quark-literal-block', par.get('class'))


class TestCreateTranslatorClass(unittest.TestCase):
    CLS = HTMLTranslator

    def test_no_enabled_no_disabled(self):
        cls = create_translator_class(self.CLS, [], [])
        self.assertTrue(issubclass(cls, self.CLS))
        self.assertFalse(issubclass(cls, HTMLCompatMixin))
        self.assertFalse(issubclass(cls, BoxesMixin))
        self.assertFalse(issubclass(cls, LiteralBlocksMixin))

    def test_some_enabled_no_disabled(self):
        cls = create_translator_class(self.CLS, ['compat'], [])
        self.assertTrue(issubclass(cls, self.CLS))
        self.assertTrue(issubclass(cls, HTMLCompatMixin))
        self.assertFalse(issubclass(cls, BoxesMixin))
        self.assertFalse(issubclass(cls, LiteralBlocksMixin))

    def test_no_enabled_some_disabled(self):
        cls = create_translator_class(self.CLS, [], ['boxes', 'literal_blocks'])
        self.assertTrue(issubclass(cls, self.CLS))
        self.assertFalse(issubclass(cls, HTMLCompatMixin))
        self.assertFalse(issubclass(cls, BoxesMixin))
        self.assertFalse(issubclass(cls, LiteralBlocksMixin))

    def test_some_enabled_some_disabled(self):
        cls = create_translator_class(self.CLS, ['compat', 'boxes'],
                                      ['literal_blocks', 'compat'])
        self.assertTrue(issubclass(cls, self.CLS))
        self.assertFalse(issubclass(cls, HTMLCompatMixin))
        self.assertTrue(issubclass(cls, BoxesMixin))
        self.assertFalse(issubclass(cls, LiteralBlocksMixin))


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
# This file is part of quark-sphinx-theme.
# Copyright (c) 2016 Felix Krull <f_krull@gmx.de>
# Released under the terms of the BSD license; see LICENSE.

import os
import unittest

import quark_sphinx_theme


class TestGetPath(unittest.TestCase):
    def test_get_path(self):
        p = quark_sphinx_theme.get_path()
        self.assertTrue(os.path.isdir(p))
        conf = os.path.join(p, 'quark', 'theme.conf')
        self.assertTrue(os.path.isfile(conf))


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
# This file is part of quark-sphinx-theme.
# Copyright (c) 2016 Felix Krull <f_krull@gmx.de>
# Released under the terms of the BSD license; see LICENSE.

from contextlib import contextmanager
import glob
import os
import shutil
import sys
import tempfile
import unittest
import zipfile

DIR = os.path.abspath(os.path.dirname(__file__))
sys.path.insert(0, DIR)
from util import attrs, capture_output  # noqa


@contextmanager
def cwd(dir):
    oldcwd = os.getcwd()
    os.chdir(dir)
    try:
        yield
    finally:
        os.chdir(oldcwd)


def run_setup(*args):
    argv = ['setup.py']
    argv.extend(args)
    with attrs(sys, argv=argv):
        with capture_output() as stdout:
            try:
                import setup  # noqa
            except Exception as exc:
                raise Exception('%s\n--- Output:\n%s-----' %
                                (exc, stdout.getvalue()))
            finally:
                try:
                    del sys.modules['setup']
                except Exception:
                    pass


SDIST_FILE_LIST = [
    ('src', [
        ('quark_sphinx_theme', [
            '__init__.py',
            '__version__.py',
            '_lovelace.py',
            '_mixin.py',
            ('ext', [
                '__init__.py',
                'html_compat.py',
                ('html_rewrite', [
                    '__init__.py',
                    'boxes.py',
                    'compat.py',
                    'literal_blocks.py',
                ]),
            ]),
            ('quark', [
                'domainindex.html',
                'genindex-single.html',
                'genindex.html',
                'layout.html',
                'theme.conf',
                'static/quark.css_t',
            ]),
        ]),
    ]),
    ('test', [
        '__init__.py',
        'test_core.py',
        'test_html_rewrite.py',
        'test_import.py',
        'test_mixin.py',
        'test_sdist.py',
        'test_theme.py',
        'util.py',
        ('testdoc-html_rewrite', [
            'conf.py',
            'contents.rst',
            'test_admonition.rst',
            'test_citation.rst',
            'test_footnote.rst',
            'test_literal_block.rst',
            'test_pre_block.rst',
            'test_sidebar.rst',
            'test_topic.rst',
            'test_warning.rst',
        ]),
        ('testdoc-theme', [
            'conf.py',
            'contents.rst',
        ]),
    ]),
    'LICENSE',
    'MANIFEST.in',
    'README.rst',
    'setup.cfg',
    'setup.py',
    'tox.ini',
]


def flatten_file_list(ls):
    for item in ls:
        if isinstance(item, str):
            yield item
        else:
            prefix, subls = item
            for subitem in flatten_file_list(subls):
                yield '%s/%s' % (prefix, subitem)


def verify_zipfile(z, ls, p=1):
    with zipfile.ZipFile(z, mode='r') as arc:
        names = [n.split('/', p)[-1] for n in arc.namelist()]
        for f in flatten_file_list(ls):
            if f not in names:
                yield f


class TestSourceDistContents(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        cls.distdir = tempfile.mkdtemp(prefix='tmp-test-sdist')
        try:
            with cwd(os.path.join(DIR, os.pardir)):
                run_setup('egg_info', '--egg-base', cls.distdir,
                          'sdist', '--formats=zip', '--dist-dir', cls.distdir)
            cls.sdist_name = glob.glob(os.path.join(cls.distdir,
                                       'quark-sphinx-theme-*.zip'))[0]
        except:
            cls.tearDownClass()
            raise

    @classmethod
    def tearDownClass(cls):
        if cls.distdir:
            shutil.rmtree(cls.distdir, True)
            cls.distdir = None

    def test_sdist_contents(self):
        missing = list(verify_zipfile(self.sdist_name, SDIST_FILE_LIST))
        if len(missing) != 0:
            self.fail('Missing expected sdist contents:\n' + '\n'.join(missing))

    def test_sdist_no_pyc(self):
        with zipfile.ZipFile(self.sdist_name, mode='r') as arc:
            for name in arc.namelist():
                if name.endswith('.pyc') or name.endswith('.pyo'):
                    self.fail('undesired file in sdist: %s' % name)


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
# This file is part of quark-sphinx-theme.
# Copyright (c) 2016 Felix Krull <f_krull@gmx.de>
# Released under the terms of the BSD license; see LICENSE.

import os
import sys
import unittest

DIR = os.path.abspath(os.path.dirname(__file__))
sys.path.insert(0, DIR)
from util import SphinxBuildFixture, with_document  # noqa


testdoc_theme = os.path.join(DIR, 'testdoc-theme')


class TestThemeDefaults(SphinxBuildFixture, unittest.TestCase):
    source_dir = testdoc_theme

    def test_quark_css(self):
        self.assertSphinxCSSValid('quark.css')

    def _nav_elems(self, doc):
        self.assertHasElement(doc, './/table[@class="nav-sidebar"]')
        self.assertHasElement(doc, './/table[@class="navbar navbar-top"]')
        self.assertHasElement(doc, './/table[@class="navbar navbar-bottom"]')

    @with_document('contents')
    def test_contents_nav_elements(self, doc):
        self._nav_elems(doc)

    @with_document('genindex')
    def test_genindex_nav_elements(self, doc):
        self._nav_elems(doc)

    @with_document('genindex-A')
    def test_genindex_a_nav_elements(self, doc):
        self._nav_elems(doc)

    @with_document('genindex-all')
    def test_genindex_all_nav_elements(self, doc):
        self._nav_elems(doc)

    @with_document('py-modindex')
    def test_domainindex_nav_elements(self, doc):
        self._nav_elems(doc)

    @with_document('search')
    def test_search_nav_elements(self, doc):
        self._nav_elems(doc)


class TestThemeExtraCss(SphinxBuildFixture, unittest.TestCase):
    source_dir = testdoc_theme
    config = {
        'html_theme_options.extra_css_files': '_static/extra1.css, '
                                              '_static/extra2.css'
    }

    def test_quark_css(self):
        self.assertSphinxCSSValid('quark.css')

    @with_document('contents')
    def test_contents_css_files(self, doc):
        extra_css = {
            s.strip() for s in
            self.config['html_theme_options.extra_css_files'].split(',')
        }
        for css in doc.findall('head/link[@rel="stylesheet"]'):
            extra_css.discard(css.get('href').strip())
        self.assertSetEqual(extra_css, set())


class TestThemeNoSidebar(SphinxBuildFixture, unittest.TestCase):
    source_dir = testdoc_theme
    config = {
        'html_theme_options.nosidebar': True,
    }

    def test_quark_css(self):
        self.assertSphinxCSSValid('quark.css')

    def _nav_elems(self, doc):
        self.assertNotElement(doc, './/table[@class="nav-sidebar"]')
        self.assertHasElement(doc, './/table[@class="navbar navbar-top"]')
        self.assertHasElement(doc, './/table[@class="navbar navbar-bottom"]')

    @with_document('contents')
    def test_contents_nav_elements(self, doc):
        self._nav_elems(doc)

    @with_document('genindex')
    def test_genindex_nav_elements(self, doc):
        self._nav_elems(doc)

    @with_document('genindex-A')
    def test_genindex_a_nav_elements(self, doc):
        self._nav_elems(doc)

    @with_document('genindex-all')
    def test_genindex_all_nav_elements(self, doc):
        self._nav_elems(doc)

    @with_document('py-modindex')
    def test_domainindex_nav_elements(self, doc):
        self._nav_elems(doc)

    @with_document('search')
    def test_search_nav_elements(self, doc):
        self._nav_elems(doc)


class TestThemeAllSettingsUnset(SphinxBuildFixture, unittest.TestCase):
    source_dir = testdoc_theme
    _SETTINGS = [
        'text_color',
        'body_font',
        'code_font',
        'body_font_size',
        'code_font_size',
        'li_extra_indent',

        'full_browser_extras',
        'extra_css_files',

        'sidebar_bgcolor',
        'sidebar_border',
        'navbar_color',
        'navbar_bgcolor',

        'link_color',
        'xref_color',
        'headerlink_color',
        'underline_links',
        'underline_navbar_links',

        'code_bgcolor',
        'pre_bgcolor',

        'admonition_border',
        'admonition_fgcolor',
        'admonition_bgcolor',
        'warning_fgcolor',
        'warning_bgcolor',
    ]
    config = {'html_theme_options.%s' % s: '' for s in _SETTINGS}

    def test_quark_css(self):
        self.assertSphinxCSSValid('quark.css')


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
# This file is part of quark-sphinx-theme.
# Copyright (c) 2016 Felix Krull <f_krull@gmx.de>
# Released under the terms of the BSD license; see LICENSE.


class BoxesMixin(object):
    __name_tag__ = 'Boxes'

    def __init__(self, *args, **kwargs):
        pass

    def visit_admonition(self, node, name=''):
        self.body.append('<table class="-x-quark-box -x-quark-admonition %s">'
                         '<tbody><tr>'
                         '<td width="100%%" class="-x-quark-box-td">'
                         % ('-x-quark-%s' % name if name else ''))
        self.__super__.visit_admonition(self, node, name)

    def depart_admonition(self, node=None):
        self.__super__.depart_admonition(self, node)
        self.body.append("</td></tr></tbody></table>")

    def visit_topic(self, node):
        self.body.append('<table class="-x-quark-box -x-quark-topic">'
                         '<tbody><tr>'
                         '<td width="100%" class="-x-quark-box-td">')
        self.__super__.visit_topic(self, node)

    def depart_topic(self, node):
        self.__super__.depart_topic(self, node)
        self.body.append('</td></tr></tbody></table>')

    def visit_sidebar(self, node):
        self.body.append('<table class="-x-quark-box -x-quark-sidebar">'
                         '<tbody><tr>'
                         '<td width="35%" class="-x-quark-box-td">')
        self.__super__.visit_sidebar(self, node)

    def depart_sidebar(self, node):
        self.__super__.depart_sidebar(self, node)
        self.body.append('</td></tr></tbody></table>')



# -*- coding: utf-8 -*-
# This file is part of quark-sphinx-theme.
# Copyright (c) 2016 Felix Krull <f_krull@gmx.de>
# Released under the terms of the BSD license; see LICENSE.


"""
Support module to build a class from a base class and one or several mixins.

A ``mixin`` is simply a class defining any number of methods. When mixed into a
base class, methods in the mixin override any methods of the same name in the
base class. When specifying multiple mixins, they are assumed to be in
descending order of precedence, i.e. earlier mixins override later ones.

The base class is available in the ``__super__`` attribute on the final combined
class. Overridden methods can use it to delegate to the base class method.
Delegating to another mixin is not supported.

Mixin classes may optionally define a class ``__name_prefix`` attribute. When
combining classes, this value will be incorporated into the name.
"""


def _name_tag(mixin):
    if hasattr(mixin, '__name_tag__'):
        return mixin.__name_tag__
    else:
        return mixin.__name__


class _NewStyleBase(object):
    """Helper class to make sure all compound classes include ``object``."""

    __name_tag__ = ''

    def __init__(self, *args, **kwargs):
        object.__init__(self)


def create_compound_class(base_class, mixins, name=None):
    """Create a combined class from a base class and several mixins.

    The elements of the ``mixin`` sequence are mixed into the base class; the
    first element in the sequence has the highest priority. If ``name`` is
    supplied, it sets the name of the final class, otherwise a name is derived
    from the base class and mixins.
    """
    all_bases = list(mixins)
    all_bases.append(base_class)
    if not issubclass(base_class, object):
        all_bases.append(_NewStyleBase)
    clsname = name if name else '_'.join(_name_tag(base) for base in all_bases
                                         if _name_tag(base))

    def __init__(self, *args, **kwargs):
        for base in reversed(all_bases):
            if hasattr(base, '__init__'):
                base.__init__(self, *args, **kwargs)

    cls_dict = {
        '__init__': __init__,
        '__super__': base_class,
    }

    return type(clsname, tuple(all_bases), cls_dict)



# -*- coding: utf-8 -*-
# This file is part of quark-sphinx-theme.
# Copyright (c) 2016 Felix Krull <f_krull@gmx.de>
# Released under the terms of the BSD license; see LICENSE.

from .html_rewrite import setup

__all__ = ['setup']



# -*- coding: utf-8 -*-
# This file is part of quark-sphinx-theme.
# Copyright (c) 2016 Felix Krull <f_krull@gmx.de>
# Released under the terms of the BSD license; see LICENSE.

import unittest

from quark_sphinx_theme import _mixin


class MixinA:
    def __init__(self, *args, **kwargs):
        pass

    def func_a(self):
        return 'MixinA:func_a'

    def func_b(self):
        return 'MixinA:func_b'


class MixinB(object):
    test_value = None

    __name_tag__ = 'BTag'

    def __init__(self, value):
        self.test_value = value

    def func_a(self):
        return 'MixinB:func_a'

    def func_c(self):
        return 'MixinB:func_c'

    def func_test_value(self):
        return self.test_value


class MixinNoInit:
    def func_c(self):
        return 'MixinNoInit:func_c'


class BaseClass:
    def __init__(self, *args, **kwargs):
        pass

    def func_a(self):
        return 'BaseClass:func_a'

    def func_b(self):
        return 'BaseClass:func_b'

    def func_c(self):
        return 'BaseClass:func_c'

    def func_d(self):
        return 'BaseClass:func_d'


class BaseClassNameTag:
    __name_tag__ = 'BaseTag'


class MixinNoTag:
    __name_tag__ = ''


class TestMixins(unittest.TestCase):
    def assertBases(self, cls, *bases):
        for base in bases:
            self.assertTrue(issubclass(cls, base))

    def test_no_mixins(self):
        cls = _mixin.create_compound_class(BaseClass, [])
        self.assertBases(cls, BaseClass, object)
        self.assertEqual(cls.__name__, 'BaseClass')
        inst = cls()
        self.assertEqual(inst.func_a(), 'BaseClass:func_a')
        self.assertEqual(inst.func_b(), 'BaseClass:func_b')
        self.assertEqual(inst.func_c(), 'BaseClass:func_c')
        self.assertEqual(inst.func_d(), 'BaseClass:func_d')

    def test_one_mixin(self):
        cls = _mixin.create_compound_class(BaseClass, [MixinB])
        self.assertBases(cls, BaseClass, MixinB, object)
        self.assertEqual(cls.__name__, 'BTag_BaseClass')
        inst = cls('test_value')
        self.assertEqual(inst.func_a(), 'MixinB:func_a')
        self.assertEqual(inst.func_b(), 'BaseClass:func_b')
        self.assertEqual(inst.func_c(), 'MixinB:func_c')
        self.assertEqual(inst.func_d(), 'BaseClass:func_d')
        self.assertEqual(inst.func_test_value(), 'test_value')

    def test_two_mixins(self):
        cls = _mixin.create_compound_class(BaseClass, [MixinA, MixinB])
        self.assertBases(cls, BaseClass, MixinA, MixinB, object)
        self.assertEqual(cls.__name__, 'MixinA_BTag_BaseClass')
        inst = cls('test_value')
        self.assertEqual(inst.func_a(), 'MixinA:func_a')
        self.assertEqual(inst.func_b(), 'MixinA:func_b')
        self.assertEqual(inst.func_c(), 'MixinB:func_c')
        self.assertEqual(inst.func_d(), 'BaseClass:func_d')
        self.assertEqual(inst.func_test_value(), 'test_value')

    def test_mixin_no_init(self):
        cls = _mixin.create_compound_class(BaseClass, [MixinNoInit])
        self.assertBases(cls, BaseClass, MixinNoInit, object)
        self.assertEqual(cls.__name__, 'MixinNoInit_BaseClass')
        inst = cls()
        self.assertEqual(inst.func_c(), 'MixinNoInit:func_c')

    def test_base_class_name_tag(self):
        cls = _mixin.create_compound_class(BaseClassNameTag, [MixinB])
        self.assertBases(cls, BaseClassNameTag, MixinB, object)
        self.assertEqual(cls.__name__, 'BTag_BaseTag')

    def test_mixin_no_tag(self):
        cls = _mixin.create_compound_class(BaseClass, [MixinA, MixinNoTag])
        self.assertBases(cls, BaseClass, MixinA, MixinNoTag, object)
        self.assertEqual(cls.__name__, 'MixinA_BaseClass')

    def test_explicit_name(self):
        cls = _mixin.create_compound_class(BaseClass, [], 'ExplicitName')
        self.assertEqual(cls.__name__, 'ExplicitName')


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
from io import open
import os
import sys
from setuptools import setup, find_packages

dist_dir = os.path.dirname(os.path.abspath(__file__))
readme = os.path.join(dist_dir, 'README.rst')
with open(readme, 'r', encoding='utf-8') as f:
    long_description = f.read()

sys.path.insert(0, os.path.join(dist_dir, 'src', 'quark_sphinx_theme'))
from __version__ import __version__  # noqa
del sys.path[0]

setup(
    name='quark-sphinx-theme',
    version=__version__,
    description='A Sphinx theme designed for QTextBrowser',
    long_description=long_description,
    url='https://bitbucket.org/fk/quark-sphinx-theme',
    author='Felix Krull',
    author_email='f_krull@gmx.de',
    license='BSD',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Framework :: Sphinx :: Theme',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 3',
        'Topic :: Documentation :: Sphinx',
        'Topic :: Software Development :: Documentation',
    ],
    zip_safe=False,
    install_requires=['sphinx>=1.1'],
    python_requires='~=2.7, ~=3.3',

    packages=find_packages('src'),
    package_dir={'': 'src'},
    include_package_data=True,
    entry_points={
        'sphinx_themes': [
            'path = quark_sphinx_theme:get_path',
        ]
    },

    test_suite='test',
    tests_require=['html5lib', 'tinycss'],
)



# -*- coding: utf-8 -*-
# This file is part of quark-sphinx-theme.
# Copyright (c) 2016 Felix Krull <f_krull@gmx.de>
# Released under the terms of the BSD license; see LICENSE.

from pygments.formatters import HtmlFormatter


class CompatHtmlFormatter(HtmlFormatter):
    def _wrap_pre(self, inner):
        # TODO: Yeah, this is pretty awkward.
        for i, s in HtmlFormatter._wrap_pre(self, inner):
            yield i, s.replace('<span></span>', '')


class HTMLCompatMixin(object):
    __name_tag__ = "Compat"

    def __init__(self, *args, **kwargs):
        self.highlighter.formatter = CompatHtmlFormatter

    def visit_citation(self, node):
        no_id_node = node.copy()
        no_id_node.delattr('ids')
        self.body.append('<div id="%s" class="-x-quark-citation-wrapper">'
                         % node.get('ids')[0])
        self.__super__.visit_citation(self, no_id_node)

    def depart_citation(self, node):
        self.__super__.depart_citation(self, node)
        self.body.append('</div>')

    def visit_footnote(self, node):
        no_id_node = node.copy()
        no_id_node.delattr('ids')
        self.body.append('<div id="%s" class="-x-quark-footnote-wrapper">'
                         % node.get('ids')[0])
        self.__super__.visit_footnote(self, no_id_node)

    def depart_footnote(self, node):
        self.__super__.depart_footnote(self, node)
        self.body.append('</div>')
